VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Registry"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = True
Option Compare Database
Option Explicit

Private Const strCLASS_NAME As String = "Registry"
Private Const lngERR_INVALID_PARAMETER As Long = 65001
Private Const lngREG_ERROR As Long = 65079
Private Const lngERR_BUFFER_TOO_SMALL As Long = 65080


' --- BEGIN --- Deklarationen fr Registry-Zugriffe --- Philipp Stiefel 2001/08/23

Private Const SYNCHRONIZE = &H100000

Private Const READ_CONTROL = &H20000
Private Const READ_WRITE = 2

Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)

Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_EVENT = &H1
Private Const KEY_NOTIFY = &H10
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Private Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))
Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
Private Const DELETE As Long = &H10000

Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_DYN_DATA = &H80000006
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_USERS = &H80000003

Private Const REG_BINARY = 3
Private Const REG_DWORD = 4
Private Const REG_CREATED_NEW_KEY = &H1
Private Const REG_DWORD_BIG_ENDIAN = 5
Private Const REG_DWORD_LITTLE_ENDIAN = 4
Private Const REG_EXPAND_SZ = 2
Private Const REG_FULL_RESOURCE_DESCRIPTOR = 9
Private Const REG_LINK = 6
Private Const REG_MULTI_SZ = 7
Private Const REG_NONE = 0
Private Const REG_NOTIFY_CHANGE_ATTRIBUTES = &H2
Private Const REG_NOTIFY_CHANGE_LAST_SET = &H4
Private Const REG_NOTIFY_CHANGE_NAME = &H1
Private Const REG_NOTIFY_CHANGE_SECURITY = &H8
Private Const REG_OPENED_EXISTING_KEY = &H2
Private Const REG_OPTION_BACKUP_RESTORE = 4
Private Const REG_OPTION_CREATE_LINK = 2
Private Const REG_OPTION_NON_VOLATILE = 0
Private Const REG_OPTION_RESERVED = 0
Private Const REG_OPTION_VOLATILE = 1
Private Const REG_REFRESH_HIVE = &H2
Private Const REG_RESOURCE_LIST = 8
Private Const REG_RESOURCE_REQUIREMENTS_LIST = 10
Private Const REG_SZ = 1
Private Const REG_WHOLE_HIVE_VOLATILE = &H1
Private Const REG_LEGAL_CHANGE_FILTER = (REG_NOTIFY_CHANGE_NAME Or REG_NOTIFY_CHANGE_ATTRIBUTES Or REG_NOTIFY_CHANGE_LAST_SET Or REG_NOTIFY_CHANGE_SECURITY)
Private Const REG_LEGAL_OPTION = (REG_OPTION_RESERVED Or REG_OPTION_NON_VOLATILE Or REG_OPTION_VOLATILE Or REG_OPTION_CREATE_LINK Or REG_OPTION_BACKUP_RESTORE)

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_FILE_NOT_FOUND As Long = &H2


Private Declare PtrSafe Function apiRegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" (ByVal hKey As LongPtr, ByVal lpSubKey As String, ByVal ulOptions As Long, _
                                                                                    ByVal samDesired As Long, phkResult As LongPtr) As Long
Private Declare PtrSafe Function apiRegCloseKey Lib "advapi32.dll" Alias "RegCloseKey" (ByVal hKey As LongPtr) As Long
Private Declare PtrSafe Function apiRegOpenKey Lib "advapi32.dll" Alias "RegOpenKeyA" (ByVal hKey As LongPtr, ByVal lpSubKey As String, phkResult As LongPtr) As Long
Private Declare PtrSafe Function apiRegSetValue Lib "advapi32.dll" Alias "RegSetValueA" (ByVal hKey As LongPtr, ByVal lpSubKey As String, ByVal dwType As Long, _
                                                                                        ByRef lpData As Any, ByVal cbData As Long) As Long
Private Declare PtrSafe Function apiRegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" (ByVal hKey As LongPtr, ByVal lpValueName As String, ByVal Reserved As Long, _
                                                                                            ByVal dwType As Long, ByRef lpData As Any, ByVal cbData As Long) As Long          ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare PtrSafe Function apiRegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" (ByVal hKey As LongPtr, ByVal lpValueName As String, ByVal lpReserved As Long, _
                                                                                            lpType As Long, ByRef lpData As Any, lpcbData As Long) As Long           ' Note that if you declare the lpData parameter as String, you must pass it By Value.
'Private Declare Function apiRegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" (ByVal hKey As Long, ByVal lpSubKey As String, ByVal Reserved As Long, ByVal lpClass As String, ByVal dwOptions As Long, ByVal samDesired As Long, lpSecurityAttributes As SECURITY_ATTRIBUTES, phkResult As Long, lpdwDisposition As Long) As Long
Private Declare PtrSafe Function apiRegCreateKey Lib "advapi32.dll" Alias "RegCreateKeyA" (ByVal hKey As LongPtr, ByVal lpSubKey As String, ByRef phkResult As LongPtr) As Long
Private Declare PtrSafe Function apiRegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" (ByVal hKey As LongPtr, ByVal lpSubKey As String) As Long

Private Declare PtrSafe Function apiRegEnumKeyExA Lib "advapi32.dll" Alias "RegEnumKeyExA" _
                                                    (ByVal hKey As LongPtr, _
                                                    ByVal dwIndex As Long, _
                                                    ByVal lpName As String, _
                                                    ByRef lpcchName As Long, _
                                                    ByVal lpReserved As Long, _
                                                    ByVal lpClass As String, _
                                                    ByRef lpcchClass As Long, _
                                                    ByVal lpftLastWriteTime As LongPtr _
                                        ) As Long

Private Declare PtrSafe Function apiRegDeleteValueA Lib "advapi32.dll" Alias "RegDeleteValueA" (ByVal hKey As LongPtr, ByVal lpValueName As String) As Long


' --- END --- Deklarationen fr Registry-Zugriffe --- Philipp Stiefel 2001/08/23

Private Type regKeyInfo
    Branch As Long
    LocalKeyName As String
End Type

Public Function ReadValue(ByVal strKeyName As String, Optional ByVal strValueName As String = "") As Variant

    Const BUFFER_SIZE As Long = 1024
    
    Const VALUE_NOT_FOUND As Long = 1208

    Dim varValue                As Variant
    Dim Keyhandle            As LongPtr
    Dim lngDataLen              As Long
    Dim lngRetVal               As Long
    Dim lngValueType            As Long
    Dim strBuffer               As String
    Dim lngBuffer               As Long
    
    Dim key As regKeyInfo
    key = getRegistryKeyInfo(strKeyName)
    
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, KEY_QUERY_VALUE, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        lngRetVal = apiRegQueryValueEx(Keyhandle, strValueName, 0, lngValueType, ByVal 0, 4)
        If lngRetVal = ERROR_FILE_NOT_FOUND Then
            Err.Raise lngERR_INVALID_PARAMETER, strCLASS_NAME & ".ReadValue", "registry value " & strValueName & " not found"
        End If
        
        Select Case lngValueType
            Case REG_SZ
                strBuffer = String(BUFFER_SIZE, vbNullChar)
                lngDataLen = Len(strBuffer)
                lngRetVal = apiRegQueryValueEx(Keyhandle, strValueName, 0, 0, ByVal strBuffer, lngDataLen)
                
                If lngDataLen > 0 Then
                    varValue = Left(strBuffer, lngDataLen)
                    If Right(varValue, 1) = vbNullChar Then
                        varValue = Left(strBuffer, lngDataLen - 1)
                    End If
                ElseIf lngDataLen > BUFFER_SIZE Then
                    Err.Raise lngERR_BUFFER_TOO_SMALL, strCLASS_NAME, "Buffer too small"
                Else
                    varValue = ""
                End If
            
            Case REG_DWORD
                lngDataLen = 4
                
                lngRetVal = apiRegQueryValueEx(Keyhandle, strValueName, 0, 0, lngBuffer, lngDataLen)
                
                varValue = lngBuffer
            Case VALUE_NOT_FOUND
                Err.Raise lngERR_INVALID_PARAMETER, strCLASS_NAME & ".ReadValue", "registry value " & strValueName & " not found"
            Case Else
                Err.Raise lngERR_INVALID_PARAMETER, strCLASS_NAME & ".ReadValue", "registry datatype not supported"
            End Select
        lngRetVal = apiRegCloseKey(Keyhandle)
    End If

    ReadValue = varValue

End Function


Public Sub WriteString(ByVal strKeyName As String, ByVal strValue As String, Optional ByVal strValueName As String = "")
    Call WriteValue(strKeyName, strValue, strValueName, REG_SZ)
End Sub

Public Sub WriteValue(ByVal strKeyName As String, ByVal varValue As Variant, Optional ByVal strValueName As String = "", Optional ByVal valueType As Long = REG_SZ)
    
    Dim Keyhandle            As LongPtr
    Dim lngDataLen              As Long
    Dim lngRetVal               As Long
    Dim strBuffer               As String
    Dim lngBuffer               As Long
    
    
    Dim key As regKeyInfo
    key = getRegistryKeyInfo(strKeyName)
    
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, KEY_SET_VALUE, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        Select Case valueType
            Case REG_SZ
                strBuffer = varValue
                lngDataLen = Len(strBuffer)

                lngRetVal = apiRegSetValueEx(Keyhandle, strValueName, 0, valueType, ByVal strBuffer, lngDataLen)
            Case REG_DWORD
                lngBuffer = CLng(varValue)
                lngDataLen = 4
                lngRetVal = apiRegSetValueEx(Keyhandle, strValueName, 0, valueType, lngBuffer, lngDataLen)
            Case Else
                Err.Raise lngERR_INVALID_PARAMETER, strCLASS_NAME & ".WriteValue", "registry datatype not supported"
            End Select
        lngRetVal = apiRegCloseKey(Keyhandle)
    End If
End Sub



Public Function KeyExists(ByVal strKeyName As String)

    Dim Keyhandle    As LongPtr
    Dim lngRetVal       As Long
    Dim blnRetVal       As Boolean
    
    Dim key As regKeyInfo
    key = getRegistryKeyInfo(strKeyName)
    
    blnRetVal = False
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, KEY_QUERY_VALUE, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        blnRetVal = True
        lngRetVal = apiRegCloseKey(Keyhandle)
    End If

    KeyExists = blnRetVal

End Function

Public Sub CreateKey(ByVal strKeyName As String)

    Dim Keyhandle    As LongPtr
    Dim lngRetVal       As Long
    Dim blnRetVal       As Boolean
    
    Dim key As regKeyInfo
    key = getRegistryKeyInfo(strKeyName)
    
    blnRetVal = False
    lngRetVal = apiRegCreateKey(key.Branch, key.LocalKeyName, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        lngRetVal = apiRegCloseKey(Keyhandle)
    Else
        Err.Raise 9999, strCLASS_NAME & ".CreateKey", "failed to create key"
    End If

End Sub

Public Sub DeleteKey(ByVal strKeyName As String)

    Dim Keyhandle    As LongPtr
    Dim lngRetVal       As Long
    Dim blnRetVal       As Boolean
    Dim lngLastBackSlash    As Long
    Dim strSubKey           As String
    Dim key             As regKeyInfo
    
    lngLastBackSlash = InStrRev(strKeyName, "\")
    strSubKey = Mid(strKeyName, lngLastBackSlash + 1)
    strKeyName = Left(strKeyName, lngLastBackSlash - 1)
    
    key = getRegistryKeyInfo(strKeyName)
    
    blnRetVal = False
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, DELETE, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        blnRetVal = True
        lngRetVal = apiRegDeleteKey(Keyhandle, strSubKey)
    End If

End Sub

Public Sub DeleteValue(ByVal KeyName As String, Optional ByVal ValueName As String = "")

    Dim Keyhandle           As LongPtr
    Dim lngRetVal           As Long
    Dim blnRetVal           As Boolean
    Dim lngLastBackSlash    As Long
    Dim strSubKey           As String
    Dim key                 As regKeyInfo
    
    key = getRegistryKeyInfo(KeyName)
    
    blnRetVal = False
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, KEY_SET_VALUE, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        lngRetVal = apiRegDeleteValueA(Keyhandle, ValueName)
        lngRetVal = apiRegCloseKey(Keyhandle)
    End If

End Sub



Private Function getRegistryKeyInfo(ByVal strFullKeyName As String) As regKeyInfo
 
    Dim strRoot             As String
    Dim lngRetVal           As Long
    Dim retVal              As regKeyInfo
    Dim lngFirstBackslash   As Long
 
    lngFirstBackslash = InStr(strFullKeyName, "\")
    
    If lngFirstBackslash > 0 Then
        strRoot = Left(strFullKeyName, lngFirstBackslash - 1)
        retVal.LocalKeyName = Mid(strFullKeyName, lngFirstBackslash + 1)
    Else
        strRoot = strFullKeyName
        retVal.LocalKeyName = ""
    End If
    
    Select Case strRoot
        Case "HKCU", "HKEY_CURRENT_USER"
            lngRetVal = HKEY_CURRENT_USER
        Case "HKLM", "HKEY_LOCAL_MACHINE"
            lngRetVal = HKEY_LOCAL_MACHINE
        Case "HKCR", "HKEY_CLASSES_ROOT"
            lngRetVal = HKEY_CLASSES_ROOT
        Case "HKEY_USERS"
            lngRetVal = HKEY_USERS
        Case "HKEY_CURRENT_CONFIG"
            lngRetVal = HKEY_CURRENT_CONFIG
    End Select
    
    retVal.Branch = lngRetVal
    
    getRegistryKeyInfo = retVal

End Function


Public Function GetSubkeyByIndex(ByVal ParentKeyName As String, ByVal SubkeyIndex As Long) As String

    Const BUFFER_SIZE As Long = 512
    
    Const VALUE_NOT_FOUND As Long = 1208

    Dim SubKeyName              As String
    Dim Keyhandle               As LongPtr
    Dim lngDataLen              As Long
    Dim lngRetVal               As Long
    Dim lngValueType            As Long
    Dim Buffer                  As String
    Dim lngBuffer               As Long
    
    Dim key As regKeyInfo
    key = getRegistryKeyInfo(ParentKeyName)
        
    lngRetVal = apiRegOpenKeyEx(key.Branch, key.LocalKeyName, 0, KEY_ENUMERATE_SUB_KEYS, Keyhandle)
    If lngRetVal = ERROR_SUCCESS Then
        Dim currentBufferSize As Long
        currentBufferSize = BUFFER_SIZE
        Buffer = String(currentBufferSize, vbNullChar)
        
        lngRetVal = apiRegEnumKeyExA(Keyhandle, SubkeyIndex, Buffer, currentBufferSize, 0, vbNullString, vbNull, 0)
        SubKeyName = Left(Buffer, currentBufferSize)
        lngRetVal = apiRegCloseKey(Keyhandle)
    End If

    GetSubkeyByIndex = SubKeyName

End Function



